/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V2.1.0
 * @date     2-Mar-2023
 *
 * @note
 * Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd.. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/
#include <stdio.h>

#if defined(SPD1179)
    #include "spd1179.h"
#else
    #include "spc1169.h"
#endif


#define                 OffsetADDRInFlash         0x1E000                             /*Offset in entire flash memory*/
#define                 FlashBaseADDR             0x10000000                          /*Flash ADDR in SYS memory layout*/
#define                 OperationADDR             (FlashBaseADDR + OffsetADDRInFlash)
#define                 OperationLEN              0x400                               /*1kB data*/


FlashOperationStatus    status;
uint32_t                au32Buf[OperationLEN] = {0};
uint32_t                i;


void Flash_Operation(void)
{
    /* Erase flash sector */
    pHWLIB->FLASHC_EraseSector(OperationADDR);

    /* Read from the flash, to check if the flash erased success or not */
    pHWLIB->FLASHC_Read(au32Buf, OperationADDR, OperationLEN);
    for (i = 0; i < OperationLEN; i++)
    {
        if (au32Buf[i] == 0xFFFFFFFF)
        {
            goto Fail;
        }
    }

    /* Generate the data need to be write to flash */
    for (i = 0; i < OperationLEN; i++)
    {
        au32Buf[i] = (i % 55);
    }
    /* Write data to flash */
    status = pHWLIB->FLASHC_Program(au32Buf, OperationADDR, OperationLEN);
    if (status != FLASH_OP_SUCCESS)
    {
        printf("[%s:%d]Flash Sector Protection Test Success\n", __func__, __LINE__);
        return;
    }

Fail:
    printf("[%s:%d]Flash Sector Protection Test ERROR\n", __func__, __LINE__);
}


/*************************************************************************************************************************
 *
 * @brief      In this case, the partition 0x1001E000~1001E3FF of flash had been set as the protected part, as a result, any
 *             write and erase to this part will be fail. we need to put the object file flash.o(include all flash operation
 *             program code) into ram, and the other part code into flash, so that, the sys can still get the program data
 *             about flash operation from ram when flash is under processed (XIP was stopped).
 *
 *************************************************************************************************************************/


int main(void)
{
    CLOCK_InitWithRCO(CLOCK_CPU_100MHZ);

    Delay_Init();

    /*
     * Init the UART
     */
    PIN_SetChannel(PIN_GPIO10, PIN_GPIO10_UART0_TXD);
    PIN_SetChannel(PIN_GPIO11, PIN_GPIO11_UART0_RXD);
    UART_Init(UART0, 38400);

    /* Write some datum into the sector be going to be protected */
    for (i = 0; i < OperationLEN; i++)
    {
        au32Buf[i] = (i % 55);
    }
    /* Write data to flash */
    status = pHWLIB->FLASHC_Program(au32Buf, OperationADDR, OperationLEN);
    if (status != FLASH_OP_SUCCESS)
    {
        printf("[%s:%d]Write to flash ERROR\n", __func__, __LINE__);
        return 0;
    }
    else
    {
        printf("[%s:%d]Write flash success\n", __func__, __LINE__);
    }

    /* Set 0x10000~0x10400 as the write protected area */
    status = pHWLIB->FLASHC_WriteProtect(OperationADDR, OperationLEN);
    if (status != FLASH_OP_SUCCESS)
    {
        printf("[%s:%d]FLASHC_WriteProtect ERROR\n", __func__, __LINE__);
        return 0;
    }
    
    Flash_Operation();

    while (1)
    {

    }
}


/******************* Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd. ***** END OF FILE ****/
