/******************************************************************************
 * @file     data_manege.c
 * @version  V1.00
 *
 * @brief    Application for data save/access with flash
 *           This application can manage flash access efficiently and extend flash life
 *           (1) This application extend flash life(erase times) from 100K times to 1500K times
 *           (2) It provides a well designed flash algorithm to avoid erase sector frequently.
 *           (3) Easy to use : Only 3 steps
 *               <1> Users define own data structure in struct UserData_t
 *               <2> Call the initial function DATA_AccessInit and set the sector adress 
 *                   (We have prepared a valid default address for you)
 *               <3> Call DATA_Save(&DataPack_t) and  DATA_Read(&DataPack_t) to access flash
 * 
 *           Please notice the following issue
 *           (1) Max data length in UserData_t is 256 byte.
 *           (2) User defined data type in DataPack_t.UserData_t
 *           (3) Please initial DataPack_t with DATA_AccessInit() before Data save/read
 *               Be sure to assign correct Flash address(32 bit) that align with the start of the sector
 *           (4) User can use DATA_Save(&DataPack_t) and  DATA_Read(&DataPack_t) to access flash
 *           (5) We also provide a basic byte series(array) function to access flash.
 *           (6) Every 15 times save, it will cost more about 27ms to erase sector
 *           
 *
 * @note
 * Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd.. All rights reserved.
*****************************************************************************/



#include "data_manage.h"
#include<stdio.h>

struct ManagementData_t gEmulationData;

/******************************************************************************
 * @brief      Initializes the control structure that used in data management 
 *
 * @param[in]  u32StartAddr   : eeprom emulation start address, this value should
 *                              be align with 0x200.
 * @param[in]  u32EndAddr     : eeprom emulation end address.this value should
 *                              be align with 0x200.
 * @param[in]  u32UserDataSize: size of user data. 
 *
 * @return Success Flag     : SUCCESS = Init Success
 *                            ERROR = Init Error
 *
 *******************************************************************************/
ErrorStatus EEPROM_Init(uint32_t u32StartAddr,uint32_t u32EndAddr, uint32_t u32UserDataSize)
{
    ErrorStatus ret = SUCCESS;
    int32_t addr = -1;
    uint16_t sectors = (u32EndAddr - u32StartAddr) / FLASH_SECTOR_SIZE;
    uint16_t indexlinecnt = FLASH_SECTOR_SIZE / 8; /* each line is 8 bytes */

    if(u32StartAddr % FLASH_SECTOR_SIZE != 0) /*start address should be align with 0x200*/
    {
        ret = ERROR;
        return ret;
    }

    if(u32EndAddr % FLASH_SECTOR_SIZE != 0) /*end address should be align with 0x200*/
    {
        ret = ERROR;
        return ret;
    }

    if(u32EndAddr - u32StartAddr < 2 * FLASH_SECTOR_SIZE)   /*Emulation region size should be large*/
    {                                                       /*than 2 sector.*/
        ret = ERROR;
        return ret;
    }

    if(u32EndAddr > FLASH_END_ADDR)
    {
        ret = ERROR;
        return ret;
    }

    /*try to split index region and data region*/
    gEmulationData.u16IndexLineExSectors = 1;
    while(gEmulationData.u16IndexLineExSectors * indexlinecnt < (sectors - gEmulationData.u16IndexLineExSectors))
    {
        gEmulationData.u16IndexLineExSectors++;
        
        if(gEmulationData.u16IndexLineExSectors > (sectors - 1))
        {
            return ERROR;
        }
    }

    gEmulationData.u16UserDataAlignSize = u32UserDataSize + ((u32UserDataSize % 8 == 0) ? 0 : (8 - u32UserDataSize % 8));
    gEmulationData.u32IndexStartAddr = u32StartAddr;
    gEmulationData.u32DataStartAddr = u32StartAddr + gEmulationData.u16IndexLineExSectors * FLASH_SECTOR_SIZE;

    gEmulationData.u32DataEndAddr = u32EndAddr;
    gEmulationData.u16OneUserDataExSectors = u32UserDataSize / FLASH_SECTOR_SIZE + 
                                                    ((u32UserDataSize % FLASH_SECTOR_SIZE) == 0 ? 0 : 1);
    gEmulationData.u32IndexLineCnt = (gEmulationData.u32DataEndAddr - 
                            gEmulationData.u32DataStartAddr) / (gEmulationData.u16OneUserDataExSectors * FLASH_SECTOR_SIZE);

    /*clear all regions */
    for(addr = u32StartAddr; addr < u32EndAddr; addr += FLASH_SECTOR_SIZE)
    {
        pHWLIB->FLASHC_EraseSector(addr);
    }

    return ret;
}


/******************************************************************************
 * @brief      Read data index from index region sector
 *
 * @param[in]  none
 *
* @return     index : indecate the clean region
 *
 *******************************************************************************/
int32_t EERPROM_GetIdleIndex()
{
    uint32_t au32ReadBuffer[2] = {0};
    int32_t count = 0;
    BOOL found = FALSE;

    au32ReadBuffer[0] = FLASH_DURTY_FLAG << 16 | 0;
    for(count = 0; count < gEmulationData.u32IndexLineCnt; count++)
    {
        /* SPD1179 Flash is 64 bit width, it read 64 bit data out every time */
        pHWLIB->FLASHC_Read(au32ReadBuffer, gEmulationData.u32IndexStartAddr + (count * 8), 2);
        if((au32ReadBuffer[0] >> 16) != FLASH_DURTY_FLAG)
        {
            found = TRUE;
            break;
        }
    }

    if(found == FALSE)
    {
        count = 0xFFFF;
    }

    return count;
}

/******************************************************************************
 * @brief      Set data index
 *
 * @param[in]  none
 *
 * @return Success Flag     : SUCCESS = Init Success
 *                            ERROR = Init Error
 *
 *******************************************************************************/
ErrorStatus EERPROM_SetIndexDurty(uint16_t index)
{
    FlashOperationStatus EStatus;
    uint32_t buf[2] = {0};

    buf[0] = (FLASH_DURTY_FLAG << 16) | index;
    /* SPD1179 Flash is 64 bit width, it read 64 bit data out every time */
    EStatus = pHWLIB->FLASHC_Program(buf, gEmulationData.u32IndexStartAddr + (index * 8), 2);

    if(EStatus != FLASH_OP_SUCCESS)
    {
        return ERROR;  /*Error*/
    }
    else
    {
        return SUCCESS; /*SUCCESS*/
    }
}


/******************************************************************************
 * @brief      Save data into flash at corresponding position(index)
 *
 * @param[in]  UserData : Pointer point to User data struct
 *
 * @return Success Flag     : SUCCESS = Init Success
 *                            ERROR = Init Error
 *
 *******************************************************************************/
ErrorStatus EEPROM_Write(struct UserData_t* UserData)
{
    FlashOperationStatus EStatus;
    uint32_t cnt = 0;
    uint32_t index = EERPROM_GetIdleIndex();
    uint32_t words = gEmulationData.u16UserDataAlignSize / 4; //numbers of words need to read

    if(index == 0xFFFF)
    {
        /*all emulation region is durty, clean the whole index region*/
        for(cnt = 0; cnt < gEmulationData.u16IndexLineExSectors; cnt++)
        {
            pHWLIB->FLASHC_EraseSector(gEmulationData.u32IndexStartAddr + cnt * FLASH_SECTOR_SIZE);
        }
        /*clean the whole data region*/
        for(cnt = gEmulationData.u32DataStartAddr; cnt < gEmulationData.u32DataEndAddr;
                cnt += FLASH_SECTOR_SIZE)
        {
            pHWLIB->FLASHC_EraseSector(cnt);
        }
        /*try to get index again*/
        index = EERPROM_GetIdleIndex();
    }

    EStatus = pHWLIB->FLASHC_Program((uint32_t *)UserData, gEmulationData.u32DataStartAddr + 
                                                index * gEmulationData.u16OneUserDataExSectors * FLASH_SECTOR_SIZE, words);
    if(EStatus != FLASH_OP_SUCCESS)
    {
        return ERROR;  /*Error*/
    }

    if(EERPROM_SetIndexDurty(index) == ERROR)
    {
        return ERROR;  /*Error*/
    }

    return SUCCESS;
}

/******************************************************************************
 * @brief      Read user data
 *
 * @param[in]  UserData : Pointer point to memory where to hold user data
 *
 * @return Success Flag     : SUCCESS = Init Success
 *                            ERROR = Init Error
 *
 *******************************************************************************/
void EEPROM_Read(struct UserData_t* UserData)
{
    uint32_t words = gEmulationData.u16UserDataAlignSize / 4; /*numbers of words need to read*/
    uint32_t index = EERPROM_GetIdleIndex();

    if(index == 0xFFFF)
    {
        index = gEmulationData.u32IndexLineCnt;
    }

    pHWLIB->FLASHC_Read((uint32_t *)UserData, gEmulationData.u32DataStartAddr + 
                                   (index - 1) * gEmulationData.u16OneUserDataExSectors * FLASH_SECTOR_SIZE, words);
}

/*** (C) COPYRIGHT 2022 Spintrol Technology Corp. ***/
