/******************************************************************************
 * @file     iap.c
 * @brief    This file provides In-Application Programming process functions.
 * @version  V2.1.0
 * @date     2-Mar-2023
 *
 * @note
 * Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd.. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/


#include "iap.h"




/* Array stored download command data and its checksum */
static uint8_t au8CmdData[2];

/* Array stored frame code data */
static __align(4) uint8_t au8CodeData[256];

/* Array stored sub-command data */
static uint8_t au8SubCmdData[12];

/* Array stored data length */
static uint8_t au8LenData[2];

/* Frame Checksum byte */
static uint8_t u8ChecksumData;




/* Function prototype declarations */
static uint8_t IAP_CalculateChecksum(const uint8_t au8Buf[], uint32_t u32Size, uint8_t u8OptionData);
static uint32_t IAP_ConvertToInt(const uint8_t au8Buf[], uint8_t u8Size);




/*******************************************************************************
 * @brief      Read data from UART/LIN
 *
 * @param[in]  UARTx    : Select the UART module
 *             au8Buf   : Pointer to the buffer stores the data readed
 *             u32Count : The number of data to be readed
 *
 * @return     none
 *
 ******************************************************************************/
void IAP_Read(UART_REGS *UARTx, uint8_t au8Buf[], uint32_t u32Count)
{
    uint32_t i = 0U;

    for (i = 0U; i < u32Count; i++)
    {
        /* Wait until data is available in RBR or the FIFO */
        while (UART_GetStatus(UARTx, UART_STS_RX_NOT_EMPTY) == 0U)
        {
        }

        /* Read one byte data from UART peripheral */
        au8Buf[i] = (uint8_t)UART_ReadByte(UARTx);
    }
}




/*******************************************************************************
 * @brief      Write a byte data to the UART/LIN
 *
 * @param[in]  UARTx  : Select the UART module
 *             u8Data : Byte data to be written
 *
 * @return     none
 *
 ******************************************************************************/
void IAP_WriteByte(UART_REGS *UARTx, uint8_t u8Data)
{
    uint8_t u8ReadBackData = 0;

    /* Wait UART RX idle, or LIN transmitter would work abnormal */
    while (UART_GetStatus(UARTx, UART_STS_RX_BUSY) != 0U)
    {
    }

    /* Send byte data */
    UART_WriteByte(UARTx, u8Data);

    /* Wait until transmitter empty */
    while (UART_GetStatus(UARTx, UART_STS_TX_EMPTY) == 0U)
    {
    }

    /* Read back the transmited data */
    while (UART_GetStatus(UARTx, UART_STS_RX_NOT_EMPTY) == 0U)
    {
    }

    u8ReadBackData = (uint8_t)UART_ReadByte(UARTx);
    if (u8ReadBackData != u8Data)
    {
        return;
    }
}




/*******************************************************************************
 * @brief      Write data to UART
 *
 * @param[in]  UARTx    : Select the UART module
 *             au8Buf   : Pointer to the buffer stores the data to be written
 *             u32Count : The number of data to be written
 *
 * @return     none
 *
 ******************************************************************************/
void IAP_Write(UART_REGS *UARTx, const uint8_t au8Buf[], uint32_t u32Count)
{
    uint32_t i = 0U;

    uint8_t u8ReadBackData = 0;

    /* Wait UART RX idle, or LIN transmitter would work abnormal */
    while (UART_GetStatus(UARTx, UART_STS_RX_BUSY) != 0U)
    {
    }

    for (i = 0U; i < u32Count; i++)
    {
        /* Send byte data */
        UART_WriteByte(UARTx, au8Buf[i]);

        /* Wait until transmitter empty */
        while (UART_GetStatus(UARTx, UART_STS_TX_EMPTY) == 0U)
        {
        }

        /* Read back the transmited data */
        while (UART_GetStatus(UARTx, UART_STS_RX_NOT_EMPTY) == 0U)
        {
        }

        u8ReadBackData = (uint8_t)UART_ReadByte(UARTx);
        if (u8ReadBackData != au8Buf[i])
        {
            return;
        }
    }
}




/*******************************************************************************
 * @brief      Calculate the checksum of the specified array data
 *
 * @param[in]  au8Buf       : Byte array
 *             u32Size      : The byte array size
 *             u8OptionData : Optional byte data, if no, set 0x00
 *
 * @return     Checksum of the byte array
 *
 ******************************************************************************/
static uint8_t IAP_CalculateChecksum(const uint8_t au8Buf[], uint32_t u32Size, uint8_t u8OptionData)
{
    /* Variable for loop */
    uint32_t i;

    /* Variable for checksum */
    uint8_t chkByte = 0xFFU;

    /* Calculate checksum */
    chkByte ^= u8OptionData;

    for (i = 0U; i < u32Size; i++)
    {
        chkByte ^= au8Buf[i];
    }

    return chkByte;
}




/*******************************************************************************
 * @brief      Convert byte array data to integer value
 *
 * @param[in]  au8Buf : Byte array
 *             u8Size : Byte array length
 *
 * @return     none
 *
 ******************************************************************************/
static uint32_t IAP_ConvertToInt(const uint8_t au8Buf[], uint8_t u8Size)
{
    /* Loop variable */
    int32_t i = 0;

    /* Store result value */
    uint32_t u32Data = 0U;


    for (i = ((int32_t)u8Size - 1); i >= 0; i--)
    {
        u32Data <<= 8;
        u32Data |= au8Buf[i];
    }

    return u32Data;
}


/****************************************************************************//**
 * @brief      Convert byte array data to int value
 *
 * @param[in]  pu8Buf: Pointer to byte array
 *             u8Size: Byte array length
 *
 * @return     none
 *
 *******************************************************************************/
uint32_t Drv_ConvertToInt(uint8_t *pu8Buf, uint8_t u8Size)
{
    /* Loop variable */
    int32_t i = 0;

    /* Store result value */
    uint32_t u32Data = 0;

    for(i = (u8Size - 1); i >= 0; i--)
    {
        u32Data <<= 8;
        u32Data |= pu8Buf[i];
    }

    return u32Data;
}



/********************************************************************************
 * @brief      Load user application code from UART
 *
 * @param[in]  none
 *
 * @return     none
 *
 *******************************************************************************/
void IAP_LoadFromLIN(void)
{
    /* Variable for loop */
    uint32_t i = 0U;

    /* Temporary variable for checksum byte */
    uint8_t  u8TempChkByte = 0xFFU;

    /* Temporary variable for address data */
    uint32_t u32TempAddr = 0U;

    /* Temporary variable for size data */
    uint32_t u32TempSize = 0U;

    /* Enable Flash register write access */
    FLASHC_WALLOW();

    /******************************************************/
    /******************  Command Routine  *****************/
    /******************************************************/
    while (1)
    {
        IAP_Read(UART1, au8CmdData, 1U);
        
        /* Command routine */
        switch (au8CmdData[0])
        {
            case CMD_READ_MEMORY:
                /* Receive the start address, number of bytes to be read and checksum */
                IAP_Read(UART1, au8SubCmdData, 6U);

                /* Calculate checksum */
                u8TempChkByte = IAP_CalculateChecksum(au8SubCmdData, 5U, 0x00U);

                /* Validate checksum */
                if (u8TempChkByte != au8SubCmdData[5])
                {
                    /* Invalid checksum, Send NACK */
                    IAP_WriteByte(UART1, NACK);
                    break;
                }

                /* Get the start address */
                u32TempAddr = IAP_ConvertToInt(au8SubCmdData, 4U);

                /* Real number data to be read */
                u32TempSize = (uint32_t)au8SubCmdData[4] + 1U;

                /* Check start address and Read data from memory */
                if ((u32TempAddr >= FLASH_START_ADDR) && (u32TempAddr < APP_END_ADDR))
                {
                    /* Read Data from Flash */
                    pHWLIB->FLASHC_Read((uint32_t *)au8CodeData, u32TempAddr, u32TempSize / 4U);
                }
                else
                {
                    /* Invalid start address, send NACK */
                    IAP_WriteByte(UART1, NACK);
                    break;
                }

                /* Send ACK */
                IAP_WriteByte(UART1, ACK);

                /* Send data */
                IAP_Write(UART1, au8CodeData, u32TempSize);

                break;

            case CMD_WRITE_MEMORY:
                /* Receive the start address and checksum */
                IAP_Read(UART1, au8SubCmdData, 5U);

                /* Calculate checksum */
                u8TempChkByte = IAP_CalculateChecksum(au8SubCmdData, 4U, au8CmdData[0]);

                /* Validate checksum */
                if (u8TempChkByte != au8SubCmdData[4])
                {
                    /* Invalid checksum, Send NACK */
                    IAP_WriteByte(UART1, NACK);
                    break;
                }

                /* Get the start address */
                u32TempAddr = IAP_ConvertToInt(au8SubCmdData, 4U);

                /* Send ACK */
                IAP_WriteByte(UART1, ACK);


                /* Receive number of bytes to be written */
                IAP_Read(UART1, au8LenData, 1U);

                /* Frame size */
                u32TempSize = (uint32_t)au8LenData[0] + 1U;

                /* Receive code data */
                IAP_Read(UART1, au8CodeData, u32TempSize);

                /* Receive checksum */
                IAP_Read(UART1, &u8ChecksumData, 1U);

                /* Calculate checksum */
                u8TempChkByte = IAP_CalculateChecksum(au8CodeData, u32TempSize, au8LenData[0]);

                /* Validate frame checksum */
                if (u8TempChkByte != u8ChecksumData)
                {
                    IAP_WriteByte(UART1, NACK);
                    break;
                }

                /* Check address and Write data to the target memory */
                if ((u32TempAddr >= FLASH_START_ADDR) && (u32TempAddr < APP_END_ADDR))
                {
                    /* Program data to Flash */
                    pHWLIB->FLASHC_Program((uint32_t *)au8CodeData, u32TempAddr, u32TempSize / 4U);
                }
                else
                {
                    /* Invalid memory address, send NACK */
                    IAP_WriteByte(UART1, 0x11);
                    break;
                }

                /* Send ACK */
                IAP_WriteByte(UART1, ACK);

                break;

            case CMD_EXT_ERASE:

                /* Receive sub-command */
                IAP_Read(UART1, au8SubCmdData, 9U);

                /* Calculate checksum */
                u8TempChkByte = IAP_CalculateChecksum(au8SubCmdData, 8, au8CmdData[0]);

                /* Validate checksum */
                if (u8TempChkByte != au8SubCmdData[8])
                {
                    IAP_WriteByte(UART1, NACK);
                    break;
                }

                /* Get the start address for erase */
                u32TempAddr = Drv_ConvertToInt(au8SubCmdData, 4);

                /* Get the erase area size */
                u32TempSize = Drv_ConvertToInt(au8SubCmdData + 4, 4);

                /* Validate address and size */
                if((u32TempAddr < FLASH_START_ADDR) || ((u32TempAddr + u32TempSize) > APP_END_ADDR))
                {
                    IAP_WriteByte(UART1, NACK);
                    break;
                }

                /* Erase */
                for(i = u32TempAddr; i < (u32TempAddr + u32TempSize); i += FLASH_SECTOR_SIZE)
                {
                    /* Erase sector, address Sector aligned */
                    pHWLIB->FLASHC_EraseSector(i);
                }

                /* Send ACK */
                IAP_WriteByte(UART1, ACK);

                break;

            default :
                break;
        } /* For switch-case   */
    } /* For while loop */
}


/******************* Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd. ***** END OF FILE ****/
